/* dvCalibrator.c
   position/orientation calibration filter for dVISE

   ver. 1.00 (5/5/99) - first public release



                 libTrCalibr & dvCalibrator software copyright

                                Copyright 1999 
              The Board of Trustees of the University of Illinois 
                             All rights reserved 

                     libTrCalibr & dvCalibrator software
                             Volodymyr Kindratenko
                 Visualization and Virtual Environments group
               National Center for Supercomputing Applications 
                  University of Illinois at Urbana-Champaign 

libTrCalibr & dvCalibrator software [both binary and source (if released)]
(hereafter, Software) is copyrighted by The Board of Trustees of the University
of Illinois (UI), and ownership remains with the UI.

The UI grants you (hereafter, Licensee) a license to use the Software for
academic, research and internal business purposes only, without a fee. Licensee
may distribute the binary and source code (if released) to third parties
provided that the copyright notice and this statement appears on all copies and
that no charge is associated with such copies.

Licensee may make derivative works. However, if Licensee distributes any
derivative work based on or derived from the Software, then Licensee will 
(1) notify Volodymyr Kindratenko [kindr@ncsa.uiuc.edu], regarding its
    distribution of the derivative work, and 
(2) clearly notify users that such derivative work is a modified version and
    not the original Software distributed by the UI.

Any Licensee wishing to make commercial use of the Software should contact the
UI, c/o Research and Technology Management Office [rtmo@uiuc.edu], to negotiate
an appropriate license for such commercial use. Commercial use includes 
(1) integration of all or part of the source code into a product for sale or 
    license by or on behalf of Licensee to third parties, or 
(2) distribution of the binary code or source code to third parties that need 
    it to utilize a commercial product sold or licensed by or on behalf of 
    Licensee.

UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY 
PURPOSE. IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY. THE UI 
SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY THE USERS OF THIS SOFTWARE.

By using or copying this Software, Licensee agrees to abide by the copyright i
law and all other applicable laws of the U.S. including, but not limited to, 
export control laws, and the terms of this license. UI shall have the right to 
terminate this license immediately by written notice upon Licensee's breach of,
or non-compliance with, any of its terms. Licensee may be held legally 
responsible for any copyright infringement that is caused or encouraged by 
Licensee's failure to abide by the terms of this license.

Form approved by University Counsel, M.A.R., 05/10/93

*/

#include <stdio.h>

#include <dsys/dm.h>
#include <dsys/inpdev.h>

#include "calibfile.h"
#include "polyfit.h"


/* module's name */
#define CALIBRATE_MODULE_NAME "tr_calibrate"

/* global variable for duVerbose stuff */
static int moduleHandle = 0;


/* returns filter type */
char *vcinputFilterTypeFunction(void)
{
  moduleHandle = duVerbose_GetModuleHandle(CALIBRATE_MODULE_NAME);
  return (VCINPUT_FILTER_TYPE_NAME);
}


/* filter data allocation function */
int vcinputFilterAllocFunction(void **data, char *params)
{
  struct fpolynomials *fpol;
  struct calibdata cdata;

  char fname[64], tmp[256], rotunits_txt;
  int order, rotunits = RADIANS;

  duVerbose_Module(VCINPUT_VERBOSE_ALLOC, moduleHandle,
		   ("tr_calibrate alloc: called\n"));

  if (!params)
    {
      duError("tr_calibrate alloc: called without parameters. Not attaching!\n");
      return 0;
    }

  /*duVerbose_Module(VCINPUT_VERBOSE_ALLOC, moduleHandle,
    ("tr_calibrate alloc: got parameter string of %s\n", params));*/

  if (sscanf(params, "%s %i %c", fname, &order, &rotunits_txt) != 3)
    {
      duError("tr_calibrate alloc: failed to parse parameters. Not attaching!\n");
      return 0;
    }

  if (rotunits_txt == 'd') rotunits = DEGREES;
  else if (rotunits_txt != 'r')
    {
      duError("tr_calibrate alloc: unrecognizable angular units. Not attaching!\n");
      return 0;
    }

  /*duVerbose_Module(VCINPUT_VERBOSE_ALLOC, moduleHandle,
    ("tr_calibrate alloc: parsed <%s>, <%i>, and <%s>\n",
    params, order, rotunits_txt));*/

  if (!(fpol = (struct fpolynomials *)malloc(sizeof(struct fpolynomials))))
    {
      duError("tr_calibrate alloc: failed to allocate memory for polynomials. Not attaching!\n");
      return 0;
    }

  /* locating calibration file */
  strcpy(tmp, getenv("DIVISIONROOT"));
  strcat(tmp, "/etc/input/calibration/");
  strcat(tmp, fname);

  if (!readcalibfile(tmp, &cdata, rotunits))
    {
      duError("tr_calibrate alloc: Unable to read calibration table. Not attaching!\n");
      free(fpol);
      return 0;
    }

  if (!fit_initialize(&cdata, order, fpol))
    {
      duError("tr_calibrate alloc: Unable to build polynomials. Not attaching!\n");
      free(fpol);
      deletecalibdata(&cdata);
      return 0;
    }

  deletecalibdata(&cdata);

  duVerbose_Module(VCINPUT_VERBOSE_ALLOC, moduleHandle,
		   ("tr_calibrate alloc: fitting polynomials are built. Filter attached!\n"));

  *data = (void *)fpol;
  return 1;
}


/* actual filter function */
int vcinputFilterFunction(VCINPUT_SENSOR_PTR sensor, void *data)
{
  dmPoint pos;
  dmEuler rot;
  dmScale scale;

  if (!data)
    {
      duError("tr_calibrate function: given no data!\n");
      return 0;
    }

  dmPointFromMat(pos, sensor->position);
  dmScaleFromMat(scale, sensor->position);
  dmEulerFromMat(rot, sensor->position);

  fit_evaluate((struct fpolynomials *)data,
	       pos[DM_X], pos[DM_Y], pos[DM_Z],
	       rot[DM_YAW], rot[DM_PITCH], rot[DM_ROLL],
	       &pos[DM_X], &pos[DM_Y], &pos[DM_Z],
	       &rot[DM_YAW], &rot[DM_PITCH], &rot[DM_ROLL]);

  dmMatFromPointEulerScale(sensor->position, pos, rot, scale);

  return 1;
}
